#import <UIKit/UIKit.h>

@interface UIImage (DrawingBlock)

// Draws an image within a block, with a given size,
// along with a supplied CGContextRef. A CGRect with
// the image-bounded rect is supplied for convenience.
+ (UIImage *)imageWithSize:(CGSize)size drawing:(void(^)(CGContextRef, CGRect))drawingBlock;

@end

// Draws a medallion-outlined image for the supplied
// base image in the given rectangle. Looks similar
// to the OS X Lion Account Picture. Credit for this 
// goes to AQMedallionView- this is just a condensed
// Quartz call that does it all.
extern void CGContextDrawMedallionForImage(CGContextRef context, CGRect rect, UIImage *image);

// Applies light linear noise in a rectangle at a certain
// opacity. For a light noise, 0.1 - 0.2 is a good value.
extern void CGContextApplyNoise(CGContextRef context, CGRect rect, CGFloat opacity);

// Draws a conical gradient, a feature that Quartz does not
// natively support, in a given rectangle, defined by an
// array of UIColors, and an array of NSNumber locations for
// the gradient stops. The array counts must match up.
extern void CGContextDrawConicalGradient(CGContextRef context, CGRect rect, NSArray *colors, NSArray *locations);