#import "GZCoreGraphicsAdditions.h"

typedef unsigned char byte;
typedef unsigned int uint;

#define F2CC(x) ((byte)(255 * x))
#define RGBAF(r,g,b,a) (F2CC(r) << 24 | F2CC(g) << 16 | F2CC(b) << 8 | F2CC(a))
#define RGBA(r,g,b,a) ((byte)r << 24 | (byte)g << 16 | (byte)b << 8 | (byte)a)

#define RGBA_R(c) ((uint)c >> 24 & 255)
#define RGBA_G(c) ((uint)c >> 16 & 255)
#define RGBA_B(c) ((uint)c >> 8 & 255)
#define RGBA_A(c) ((uint)c >> 0 & 255)

static inline byte blerp(byte a, byte b, float w)
{
	return a + w * (b - a);
}

static inline int lerp(int a, int b, float w)
{
	return RGBA(blerp(RGBA_R(a), RGBA_R(b), w),
				blerp(RGBA_G(a), RGBA_G(b), w),
				blerp(RGBA_B(a), RGBA_B(b), w),
				blerp(RGBA_A(a), RGBA_A(b), w));
}

@implementation UIImage (DrawingBlock)

+ (UIImage *)imageWithSize:(CGSize)size drawing:(void(^)(CGContextRef, CGRect))drawingBlock
{
    UIGraphicsBeginImageContextWithOptions(size, NO, 0.0f);
    drawingBlock(UIGraphicsGetCurrentContext(), (CGRect){CGPointZero, size});
    UIImage *image = UIGraphicsGetImageFromCurrentImageContext();
    UIGraphicsEndImageContext();
    
    return image;
}

@end

void CGContextDrawMedallionForImage(CGContextRef context, CGRect drawingRect, UIImage *image) {
    CGSize imageSize = image.size;
    if(imageSize.width <= 0 || imageSize.height <= 0)
        return;
    
    UIGraphicsBeginImageContextWithOptions(imageSize, NO, 0.0f);
    CGContextRef contextRef = UIGraphicsGetCurrentContext();
    CGRect rect = CGRectMake(0, 0, imageSize.width, imageSize.height);
    
    UIColor *borderColor = [UIColor whiteColor];
//    CGFloat borderWidth = 2.0f;
    CGFloat borderWidth = 1.0f;
   
    UIColor *shadowColor = [UIColor colorWithRed:0.25f green:0.25f blue:0.25f alpha:.75f];
    CGSize shadowOffset = CGSizeMake(0, 0);
    CGFloat shadowBlur = 2.f;
    
    CGRect imageRect = CGRectMake(borderWidth, borderWidth,
                                  rect.size.width - (borderWidth * 2),
                                  rect.size.height - (borderWidth * 2));
    
    CGColorSpaceRef maskColorSpaceRef = CGColorSpaceCreateDeviceGray();
    CGContextRef mainMaskContextRef = CGBitmapContextCreate(NULL, rect.size.width, rect.size.height,
                                                            8, rect.size.width, maskColorSpaceRef, 0);
    CGContextRef shineMaskContextRef = CGBitmapContextCreate(NULL, rect.size.width, rect.size.height,
                                                             8, rect.size.width, maskColorSpaceRef, 0);
    
    CGColorSpaceRelease(maskColorSpaceRef);
    CGContextSetFillColorWithColor(mainMaskContextRef, [UIColor blackColor].CGColor);
    CGContextSetFillColorWithColor(shineMaskContextRef, [UIColor blackColor].CGColor);
    CGContextFillRect(mainMaskContextRef, rect);
    CGContextFillRect(shineMaskContextRef, rect);
    CGContextSetFillColorWithColor(mainMaskContextRef, [UIColor whiteColor].CGColor);
    CGContextSetFillColorWithColor(shineMaskContextRef, [UIColor whiteColor].CGColor);
    
    CGContextMoveToPoint(mainMaskContextRef, 0, 0);
    CGContextAddEllipseInRect(mainMaskContextRef, imageRect);
    CGContextFillPath(mainMaskContextRef);
    
    CGContextTranslateCTM(shineMaskContextRef, -(rect.size.width / 4), rect.size.height / 4 * 3);
    CGContextRotateCTM(shineMaskContextRef, -45.f);
    CGContextMoveToPoint(shineMaskContextRef, 0, 0);
    CGContextFillRect(shineMaskContextRef, CGRectMake(0, 0, rect.size.width / 8 * 5, rect.size.height));
    
    CGImageRef mainMaskImageRef = CGBitmapContextCreateImage(mainMaskContextRef);
    CGImageRef shineMaskImageRef = CGBitmapContextCreateImage(shineMaskContextRef);
    CGContextRelease(mainMaskContextRef);
    CGContextRelease(shineMaskContextRef);
    
    CGContextSaveGState(contextRef);
    CGImageRef imageRef = CGImageCreateWithMask(image.CGImage, mainMaskImageRef);
    
    CGContextSaveGState(contextRef);
    CGContextDrawImage(contextRef, rect, imageRef);
    CGContextRestoreGState(contextRef);
    CGContextSaveGState(contextRef);
    
    CGFloat colors[6] = {1.f, 0.75f, 1.f, 0.f, 0.f, 0.f};
    CGFloat colorStops[3] = {1.f, 0.35f, 0.f};
    CGColorSpaceRef grayColorSpace = CGColorSpaceCreateDeviceGray();
    CGGradientRef alphaGradient = CGGradientCreateWithColorComponents(grayColorSpace, colors, colorStops, 3);
    CGColorSpaceRelease(grayColorSpace);
    
    CGContextClipToMask(contextRef, rect, mainMaskImageRef);
    CGContextClipToMask(contextRef, rect, shineMaskImageRef);
    CGContextSetBlendMode(contextRef, kCGBlendModeLighten);
    CGContextDrawLinearGradient(contextRef, alphaGradient, CGPointMake(0, 0), CGPointMake(0, rect.size.height), 0);
    CGGradientRelease(alphaGradient);
    
    CGImageRelease(mainMaskImageRef);
    CGImageRelease(shineMaskImageRef);
    CGImageRelease(imageRef);
    CGContextRestoreGState(contextRef);
    
    CGContextSetLineWidth(contextRef, borderWidth);
    CGContextSetStrokeColorWithColor(contextRef, borderColor.CGColor);
    CGContextMoveToPoint(contextRef, 0, 0);
    CGContextAddEllipseInRect(contextRef, imageRect);
    
    CGContextSetShadowWithColor(contextRef, shadowOffset, shadowBlur, shadowColor.CGColor);
    CGContextStrokePath(contextRef);
    CGContextRestoreGState(contextRef);
    
    UIImage *output = UIGraphicsGetImageFromCurrentImageContext();
    UIGraphicsEndImageContext();
    CGContextDrawImage(context, drawingRect, output.CGImage);
}

void CGContextDrawConicalGradient(CGContextRef context, CGRect rect, NSArray *colors, NSArray *locations) {
	int w = CGRectGetWidth(rect);
	int h = CGRectGetHeight(rect);
    
	int bitsPerComponent = 8;
	int bpp = 4 * bitsPerComponent / 8;
	int byteCount = w * h * bpp;
    
	int colorCount = colors.count;
	int locationCount = 0;
	int* _colors = NULL;
	float* _locations = NULL;
    
	if(colorCount > 0) {
		_colors = calloc(colorCount, bpp);
		int *p = _colors;
        
		for(id c in colors) {
			float r, g, b, a;
            
			if(![c getRed:&r green:&g blue:&b alpha:&a]) {
				if(![c getWhite:&r alpha:&a])
					continue;
				g = b = r;
			}
			*p++ = RGBAF(r, g, b, a);
		}
	}
    
	if(locations.count > 0 && locations.count == colorCount) {
		locationCount = locations.count;
		_locations = calloc(locationCount, sizeof(_locations[0]));
        
		float *p = _locations;
		for(NSNumber *n in locations) {
			*p++ = [n floatValue];
		}
	}
    
	byte* data = malloc(byteCount);
    if(colorCount > 0 && locationCount > 0 && locationCount == colorCount) {
        int* p = (int *)data;
        float centerX = (float)w / 2;
        float centerY = (float)h / 2;
        
        for (int y = 0; y < h; y++)
            for (int x = 0; x < w; x++) {
                float dirX = x - centerX;
                float dirY = y - centerY;
                float angle = atan2f(dirY, dirX);
                
                if(dirY < 0)
                    angle += 2 * M_PI;
                angle /= 2 * M_PI;
                
                int index = 0, nextIndex = 0;
                float t = 0;
                
                if(locationCount > 0) {
                    for(index = locationCount - 1; index >= 0; index--) {
                        if(angle >= _locations[index])
                            break;
                    }
                    
                    if(index >= locationCount)
                        index = locationCount - 1;
                    nextIndex = index + 1;
                    if(nextIndex >= locationCount)
                        nextIndex = locationCount - 1;
                    
                    float ld = _locations[nextIndex] - _locations[index];
                    t = ld <= 0 ? 0 : (angle - _locations[index]) / ld;
                } else {
                    t = angle * (colorCount - 1);
                    index = t;
                    t -= index;
                    
                    nextIndex = index + 1;
                    if(nextIndex >= colorCount)
                        nextIndex = colorCount - 1;
                }
                
                int lc = _colors[index];
                int rc = _colors[nextIndex];
                int color = lerp(lc, rc, t);
                
                *p++ = color;
            }
    }
    
	if(_colors) free(_colors);
    if(_locations) free(_locations);
    
    CGColorSpaceRef colorSpace = CGColorSpaceCreateDeviceRGB();
    CGBitmapInfo bitmapInfo = kCGImageAlphaPremultipliedLast | kCGBitmapByteOrder32Little;
    
    CGContextRef ctx = CGBitmapContextCreate(data, w, h, bitsPerComponent, w * bpp, colorSpace, bitmapInfo);
    CGImageRef img = CGBitmapContextCreateImage(ctx);
    
    CGColorSpaceRelease(colorSpace);
    CGContextRelease(ctx);
    free(data);
    
    CGContextDrawImage(context, rect, img);
    CGImageRelease(img);
}

void CGContextApplyNoise(CGContextRef context, CGRect rect, CGFloat opacity)
{
    NSUInteger width = 128;
    NSUInteger height = 128;
    
    NSUInteger size = width * height;
    char *rgba = (char *)malloc(size);
    srand(124);
    
    for(NSUInteger i=0; i < size; ++i)
        rgba[i] = rand() % 256;
    
    CGColorSpaceRef colorSpace = CGColorSpaceCreateDeviceGray();
    CGContextRef bitmapContext = CGBitmapContextCreate(rgba, width, height, 8, width, colorSpace, kCGImageAlphaNone);
    CGImageRef image = CGBitmapContextCreateImage(bitmapContext);
    
    CFRelease(bitmapContext);
    CGColorSpaceRelease(colorSpace);
    free(rgba);
    
    
    CGContextSaveGState(context);
    CGContextClipToRect(context, rect);
    CGContextSetAlpha(context, opacity);
    CGContextSetBlendMode(context, kCGBlendModeScreen);
    
    CGRect imageRect = (CGRect){CGPointZero, CGImageGetWidth(image), CGImageGetHeight(image)};
    CGContextDrawTiledImage(context, imageRect, image);
    
    CGContextSetBlendMode(context, kCGBlendModeNormal);
    CGContextRestoreGState(context);
    CGImageRelease(image);
}
